library(stockassessment)
old<-read.table("data/old.dat", head=TRUE)
source("src/common.R")
# load what has been saved
setwd("run")
for(f in dir(pattern="RData"))load(f) 
setwd("..")

basefit<-NULL
if(file.exists("baserun/model.RData")){
  local({load("baserun/model.RData"); basefit<<-fit})
  if(abs(logLik(basefit)-logLik(fit))<1.0e-9)basefit<<-fit
}


plotcounter<-1
tit.list<-list()

setcap<-function(title="", caption=""){   
 tit.list[length(tit.list)+1]<<-paste("# Title",plotcounter,"\n")
 tit.list[length(tit.list)+1]<<-paste(title,"\n")
 tit.list[length(tit.list)+1]<<-paste("# Caption",plotcounter,"\n")
 tit.list[length(tit.list)+1]<<-paste(caption,"\n")
 plotcounter<<-plotcounter+1 
}


############################## plots ##############################
plots<-function(){
  par(cex.lab=1, cex.axis=1, mar=c(5,5,1,1))
    
  if(exists("fit")){
    fits <- c(fit,basefit)
    
    ssbplot(fits, addCI=TRUE)
    lines(old$Year, old$SSB, col="red", lwd=2)
    stampit()
    setcap("Spawning stock biomass", "Spawning stock biomass. 
            Estimates from the current run and point wise 95% confidence 
            intervals are shown by black line and shaded area.")
    
    fbarplot(fits, addCI=TRUE)
    lines(old$Year, old$F24, col="red", lwd=2)    
    stampit()

    recplot(fits, addCI=TRUE)
    lines(old$Year, old$Recruits, col="red", lwd=2)

    stampit()
    
    fit$sdrep$value<-c(fit$sdrep$value, fit$sdrep$par.fixed)
    fit$sdrep$sd<-c(fit$sdrep$sd, sqrt(diag(fit$sdrep$cov.fixed)))
    stockassessment:::plotit(fit, "logScale", x=fit$conf$keyScaledYears, ylab = "Scale", trans = exp)


    catchplot(fits, addCI=TRUE)
    stampit()
    
    obscorrplot(fit)
    stampit()

    
    for(f in 1:fit$data$noFleets){
      fitplot(fit, fleets=f)
      stampit()
    }
    
    # F-at-age
    matplot(fit$data$years, faytable(fit), type="l", lty=1:ncol(faytable(fit)), lwd=2, xlab="Year", ylab="F")
    legend('topleft', col=1:ncol(faytable(fit)), lty=1:ncol(faytable(fit)), legend=c(colnames(faytable(fit))[1:5],"6+"), bty='n', lwd=2)
    stampit()
    
    # Fbar landings and discards
    fbar.land <- rowMeans((faytable(fit)[-nrow(faytable(fit)),] * fit$data$landFrac)[, 2:4])
    fbar.dis <- rowMeans((faytable(fit)[-nrow(faytable(fit)),] * (1 - fit$data$landFrac))[, 2:4])
    plot(rownames(faytable(fit)[-nrow(faytable(fit)),]), fbar.land, lwd = 2, col = 'blue', type = 'l', ylim = c(0, max(fbar.land)), las = 1, xlab = 'Year', ylab = expression(bar(F)[2-4]))
    lines(rownames(faytable(fit)[-nrow(faytable(fit)),]), fbar.dis, lwd = 2, col = 'red')
    legend('topleft', lty='solid', col=c('blue','red'), legend=c('Landed', 'Discarded'), bty='n', lwd=2)
    
    

    #Q<-fit$pl$logFpar
    #Qsd<-fit$plsd$logFpar
    #key<-fit$conf$keyLogFpar
    #fun<-function(x)if(x<0){NA}else{Q[x+1]}
    #FF<-Vectorize(fun)
    #ages<-fit$conf$minAge:fit$conf$maxAge
    #matplot(ages, exp(t(matrix(FF(key), nrow=5))), type="l", lwd=5, lty="solid", xlab="Ages", ylab="Q")
    #legend("topright", lwd=5, col=2:5, legend=attr(fit$data, "fleetNames")[2:5])
    #stampit()

  }  
  
  if(exists("RES")){  
    plot(RES)
    par(mfrow=c(1,1))
    stampit()
  
  }
  
   
  if(exists("RESP")){  
    plot(RESP)
    par(mfrow=c(1,1))
    stampit()
  } 
 
  
  if(exists("LO")){  
    ssbplot(LO)
    stampit()
    
    fbarplot(LO)
    stampit()

    recplot(LO)
    stampit()

    catchplot(LO)
    stampit()
    
  } 
  
  if(exists("RETRO")){  
    ssbplot(RETRO, las=0, drop=1)
    stampit()
    
    fbarplot(RETRO, las=0, drop=1)
    stampit()
    
    recplot(RETRO, las=0, drop=1)
    stampit()

    catchplot(RETRO)
    stampit()
  } 
  if(exists("FC")){  
    lapply(FC, function(f){plot(f); title(attr(f,"label"), outer=TRUE, line=-1); stampit()})
  }  
  
}


setwd('res')
file.remove(dir(pattern='png$'))
stamp<-gsub('-[[:digit:]]{4}$','',gsub(':','.',gsub(' ','-',gsub('^[[:alpha:]]{3} ','',date()))))
png(filename = paste(stamp,"_%03d.png", sep=''), width = 480, height = 480,
    units = "px", pointsize = 10, bg = "white")
  plots()    
dev.off()

writeLines(unlist(tit.list),'titles.cfg') 

png(filename = paste("big_",stamp,"_%03d.png", sep=''), width = 1200, height = 1200, 
    units = "px", pointsize = 20, bg = "white")
  plots()    
dev.off()

#pdf(onefile=FALSE, width = 8, height = 8)
#  plots()    
#dev.off()

file.remove(dir(pattern='html$'))

tsb<-tsbtable(fit)
colnames(tsb)<-c("TSB","Low", "High")
tab.summary <- cbind(summary(fit), tsb)
xtab(tab.summary, caption=paste('Table 1. Estimated recruitment, spawning stock biomass (SSB), 
     and average fishing mortality','.',sep=''), cornername='Year', 
     file=paste(stamp,'_tab1.html',sep=''), dec=c(0,0,0,0,0,0,3,3,3,0,0,0))

ftab <- faytable(fit)
xtab(ftab, caption=paste('Table 2. Estimated fishing mortality at age','.',sep=''), cornername='Year \ Age', 
     file=paste(stamp,'_tab2.html',sep=''), dec=rep(3,ncol(ftab)))

ntab <- ntable(fit)
xtab(ntab, caption=paste('Table 3. Estimated stock numbers at age','.',sep=''), cornername='Year \ Age', 
     file=paste(stamp,'_tab3.html',sep=''), dec=rep(0,ncol(ntab)))

ptab <- partable(fit)
xtab(ptab, caption=paste('Table 4. Table of model parameters','.',sep=''), cornername='Parameter name', 
     file=paste(stamp,'_tab4.html',sep=''), dec=rep(3,ncol(ptab)))

mtab <- modeltable(c(Current=fit, base=basefit))
mdec <- c(2,0,2,6)[1:ncol(mtab)]
xtab(mtab, caption=paste('Table 5. Model fitting','.',sep=''), cornername='Model', 
     file=paste(stamp,'_tab5.html',sep=''), dec=mdec)

# Catch multipliers
scaletab <- ptab[substr(rownames(ptab),1,8)=="logScale", 3:5]
rownames(scaletab) <- 1993:2005
colnames(scaletab) <- c("Catch multiplier","Low","High")
xtab(scaletab, caption=paste('Table 6. Estimated catch scaling factors','.',sep=''), cornername='Year', 
     file=paste(stamp,'_tab6.html',sep=''), dec=rep(2,ncol(scaletab)))
     
# Catch
yy <- fit$data$years[-length(fit$data$years)]
scale <- rep(1, length(yy))
phi <- 1/exp(fit$sdrep$par.fixed[names(fit$sdrep$par.fixed)=="logScale"])
scaleYears <- fit$conf$keyScaledYears
scale[yy%in%scaleYears] <- phi

caytable<-function (fit)
{
  N <- ntable(fit)
  F <- faytable(fit)
  M <- fit$data$natMor
  F/(F+M)*(1-exp(-F-M))*N
}
cay <- caytable(fit)
ctab <- cbind(rowSums(fit$data$landFrac*cay[1:nrow(fit$data$landMeanWeight), ]          *fit$data$landMeanWeight)*scale,
              rowSums((1-fit$data$landFrac)*cay[1:nrow(fit$data$disMeanWeight), ]*fit$data$disMeanWeight*scale),
              rowSums(cay[1:nrow(fit$data$catchMeanWeight), ]*fit$data$catchMeanWeight*scale),
              rowSums(cay[1:nrow(fit$data$catchMeanWeight), ]*fit$data$catchMeanWeight))

rownames(ctab) <- yy
colnames(ctab) <- c("Landings","Discards","Catch","Total Removal")
xtab(ctab, caption=paste('Table 7. Catch table','.',sep=''), cornername='Year', 
     file=paste(stamp,'_tab7.html',sep=''), dec=rep(0,ncol(ctab)))
     
# Catch-at-age
xtab(cay, caption=paste('Table 8. Predicted catches including catch multiplier','.',sep=''), cornername='Year', 
     file=paste(stamp,'_tab8.html',sep=''), dec=rep(0,ncol(cay)))


# Observation SDs
o <- order(fit$conf$keyVarObs[fit$conf$keyVarObs>-1])
fleet <- row(fit$conf$keyVarObs)[fit$conf$keyVarObs>-1][o]
age <- col(fit$conf$keyVarObs)[fit$conf$keyVarObs>-1][o]
osdtab <- cbind(fleet,age,exp(fit$opt$par[names(fit$opt$par)=="logSdLogObs"][(fit$conf$keyVarObs+1)[(fit$conf$keyVarObs+1)>0][o]]))
colnames(osdtab) <- c("Fleet number","Age","Estimate")
rownames(osdtab) <- 1:nrow(osdtab)
xtab(osdtab, caption=paste('Table 9. Estimated observation SDs','.',sep=''), cornername='Index', 
     file=paste(stamp,'_tab9.html',sep=''), dec=c(0,0,5))

if(exists("FC")){  
    ii<-0
    lapply(FC, function(f){
       ii<<-ii+1;
       tf<-t(attr(f,"shorttab"));
       dec<-c(3,0,0,0,3,3,0,0);
       colnames(tf)<-c("fbar", "rec", "ssb", "catch", "fbarL", "fbarD", "Land", "Discard")
       xtab(tf, caption=paste0('Forecast table ',ii,'. ', attr(f,"label"),'.'), 
       cornername='Year', file=paste(stamp,'_tabX',ii,'.html',sep=''), dec=dec);      
       })
}  

setwd("..") 

